#
# To run execute following in command line: 
#
#    powershell -ExecutionPolicy Unrestricted \path\to\win32-trace.ps1
#

Param(        
    [String]$Exec = '.\stx.com',
    [ValidateScript({Test-Path -LiteralPath $_})]
    [String]$LogDirectoryPath
)

if ( (Get-Command $Exec -ErrorAction SilentlyContinue) -eq $Null ) {
    Write-Host "ERROR: '$Exec' not found."
    exit    
}

if ( (Get-Command "procmon.exe" -ErrorAction SilentlyContinue) -eq $Null ) {
    Write-Host "ERROR: 'procmon.exe' not in PATH."
    Write-Host "       Add directory with Sysinternals tools to your PATH"
    exit    
}

if ( (Get-Command "vmmap.exe" -ErrorAction SilentlyContinue) -eq $Null ) {
    Write-Host "ERROR: 'vmmap.exe' not in PATH."
    Write-Host "       Add directory with Sysinternals tools to your PATH"
    exit    
}

$Timestamp = Get-Date -UFormat "%Y-%m-%d-%H-%M-%S"
$ProgName  = "stx.com"

if ($LogDirectoryPath -ne "") {
    $LogDirectory = Get-Item -Path $LogDirectoryPath
} else {        
    $LogDirectory = New-Item -ItemType directory -Path "stx_issue_200-$timestamp"
}

$CrashScript = "$LogDirectory\stx_issue_200.$Timestamp.st"
$CrashStderr = "$LogDirectory\stx_issue_200.$Timestamp.stderr.txt"

@"
| t | 
ObjectMemory class compile:
'printHeapInfo
    <cdecl: void "__dumpSpaceInfo__" () module: "librun">    
    self primitiveFailed 
'.
t := Unicode16String new: 46. 
[ 
    1 to: 30 do:[:i | 
        Stdout nextPutLine: i printString , '-', (t size*2) printString. 
        Stderr nextPutLine: i printString , '-', (t size*2) printString. 
        Stderr flush.
        ObjectMemory printHeapInfo.
        Stderr flush.
        i == 25 ifTrue:[
            Stdout nextPutLine: 'Press enter within 10 seconds in the other window'. 
            Delay waitForSeconds: 10
        ]. 
        t := t , t 
    ] 
] on: Error do:[:ex | 
    Stdout nextPutLine: ex description.
    Smalltalk exit: 1.
]. 
Stdout nextPutLine: 'Passed'.
Smalltalk exit:0"
"@ | Out-File $CrashScript -Encoding ASCII


# procmon.exe /NoFilter /Quiet /BackingFile "$LogDirectory\$ProgName.$Timestamp.pml" 
Write-Host "Starting $ProgName..."
start $Exec -RedirectStandardError $CrashStderr -a "--load $CrashScript"
Write-Host "Press 'Enter' once  you see a message in the other window"
Write-Host "telling you to do so"
Read-Host 
vmmap -64 -p $ProgName "$LogDirectory\stx_issue_200.$Timestamp.mmp" | Out-Null
# procmon.exe /Terminate /PagingFile | Out-Null

Write-Host ""
Write-Host "All logs are saved in:"
Write-Host ""
Write-Host "    $LogDirectory"
Write-Host ""
Write-Host "Please make a .zip of it and attach it to issue #200 at:"
Write-Host ""
Write-Host "    https://swing.fit.cvut.cz/projects/stx-jv/ticket/200"
Write-Host ""
Write-Host "Press 'Enter'"
Read-Host 



